;********************************************************************
;	hex.asm - a library of functions that format and display hexa-
;	decimal numbers.
;********************************************************************


	title		Inside Assembler library functions
	subtitle	HEX -- hexadecimal number formatting and display
	.model	MEMORY_MODEL


	include macros.inc			; MASM 6.0's macro library
	include stdmac.inc			; Inside Assembler macro library


	;***********;
	; DATA AREA ;
	;***********;
	.data

hexbuff byte	'????:????$'	; used by the hexprt??? functions as
								; a temporary output buffer

	;***********;
	; CODE AREA ;
	;***********;
    .code


;--------------------------------------------------------------------
;	hexfmt$yn - format a nybble in hexadecimal into the buffer
;
;	INP:	AL - contains the nybble to format
;			DI - ptr to location to store the formatted character
;	OUT:	DI - incremented to next buffer location
;	USES:	AL
;
hexfmt$yn proc

	and 	al, 0fh 			; Discard the upper nybble
	add 	al, '0' 			; Shift range from 0..f to '0'..'?'
	cmp 	al, '9' 			; If in range of ':'..'?' shift the
	jng		@F					; range to 'A'..'F'
	add 	al, 'A'-':'
@@: mov 	[di], al
	inc 	di
	ret

hexfmt$yn endp


;--------------------------------------------------------------------
;	hexfmt$bn - format a byte in hexadecimal into the buffer
;
;	INP:	AL - contains the byte to format
;			DI - ptr to buffer to hold the formatted character
;	OUT:	DI - points to the next buffer location
;	USES:	AL
;
hexfmt$bn proc

	push	ax					; Save the lower nybble
	shr 	al, 1				; Move upper nybble to lower
	shr 	al, 1
	shr 	al, 1
	shr 	al, 1
	call	hexfmt$yn			; Format upper nybble into buffer
	pop 	ax					; Restore lower nybble
	call	hexfmt$yn			; Format lower nybble into buffer
	ret

hexfmt$bn endp


;--------------------------------------------------------------------
;	hexfmt$wn - format a word in hexadecimal into the buffer
;
;	INP:	AX - word to fomat into the buffer
;			DI - ptr to buffer to hold formatted word
;	OUT:	DI - points to next buffer location
;	USES:	AX
;
hexfmt$wn proc

	xchg	ah, al				; Swap upper byte with lower byte
	call	hexfmt$bn			; Format upper byte into buffer
	xchg	ah, al				;
	call	hexfmt$bn			; Format lower byte into buffer
	ret

hexfmt$wn endp


;--------------------------------------------------------------------
;	hexfmt$dn - format a double word in hexadecimal into the buffer
;
;	INP:	DX:AX - the double word to format into the buffer
;			DI - ptr to buffer to hold the formatted double word
;	OUT:	DI - points to next buffer location
;	USES:	AX, DX
;
hexfmt$dn proc

	xchg	dx, ax				; Preserve lower word
	call	hexfmt$wn			; Format upper word into buffer
	mov 	byte ptr [di], ':'	; Insert ':' delimiter into buffer
	inc 	di
	xchg	dx, ax				; Restore lower word
	call	hexfmt$wn			; Format lower word into buffer
	ret

hexfmt$dn endp


;--------------------------------------------------------------------
;	hexout$y - output a hexadecimal nybble to the screen
;
;	INP:	AL - the lower half is the nybble to output
;	USES:	AL
;
hexout$y proc

	@SaveRegs di, dx
	mov 	dx, offset hexbuff	; Start of hex output buffer
	mov 	di, dx				; Set pointer for hexfmt
	call	hexfmt$yn			; Format nybble into buffer
	mov 	byte ptr [di],'$'	; Terminate the string
	dosSVC	9					; Print the string
	@RestoreRegs
	ret

hexout$y endp


;--------------------------------------------------------------------
;	hexout$b - output a hexadecimal byte to the screen
;
;	INP:	AL - the byte to output
;	USES:	AX
;
hexout$b proc

	@SaveRegs	di, dx
	mov 	dx, offset hexbuff	; Start of hex output buffer
	mov 	di, dx				; Set pointer for hexfmt
	call	hexfmt$bn			; Format byte into buffer
	mov 	byte ptr [di],'$'	; Terminate the string
	dosSVC	9					; Print the string
	@RestoreRegs
	ret

hexout$b endp


;--------------------------------------------------------------------
;	hexout$w - output a hexadecimal word to the screen
;
;	INP:	AX - the word to print
;	USES:	AX
;
hexout$w proc

	@SaveRegs	di, dx
	mov 	dx,offset hexbuff	; Start of hex output buffer
	mov 	di, dx				; Set pointer for hexfmt
	call	hexfmt$wn			; Format the word into the buffer
	mov 	byte ptr [di],'$'	; Terminate the string
	dosSVC	9					; Print the string
	@RestoreRegs
	ret

hexout$w endp


;--------------------------------------------------------------------
;	hexout$d - print a double word in hexadecimal on the screen
;
;	INP:	DX:AX - the double word to print
;	USES:	AX, DX
;
hexout$d proc

	@SaveRegs	di, dx, cx
	mov 	cx, offset hexbuff	; Start of hex output buffer
	mov 	di, cx				; Pointer value for hexfmt
	call	hexfmt$dn			; Format double word into the buffer
	mov 	byte ptr [di],'$'	; Terminate the string
	mov 	dx, cx				; <we stored str in CX because
								;  hexfmt$dn destroys DX>
	dosSVC	9					; Print the string
	@RestoreRegs
	ret

hexout$d endp

    end
