;********************************************************************
;	TESTMSBN.ASM - test the floating point format conversion routines
;
;	To assemble, type the following line:
;	ml /DMEMORY_MODEL=small testmsbn.asm msbincvt.asm hex.asm
;********************************************************************

CRLF	TEXTEQU	<0dh, 0ah>

	.model MEMORY_MODEL
	.dosseg

	include MACROS.INC
	include STDMAC.INC

PR_STR macro strofs:REQ
	mov 	dx, offset strofs
	dosSVC	9
endm

PR_STR_HEX macro strofs:REQ
	PR_STR	strofs
	call	prthex
endm

	.stack

;--------------------------------------------------------------------
;	Here is a list of short and long reals that we'll convert from
;	IEEE format to Old Microsoft format, and then back again. You can
;	view them in CodeView -- see the article text
;
.data

shorts	dd	1.0000, 3.14159, 2.71828, 7.453e3, -5.12e30
longs	dq	18.000, 91.6666, 1.35e50, -2.2e16, -1.35e-9

Ishort	byte	" Error converting IEEE short real!$"
Mshort	byte	" Error converting Microsoft single!$"
Ilong	byte	" Error converting IEEE long real!$"
Mlong	byte	" Error converting Microsoft double!$"
cvtsto	byte	" converts to $"
newline byte	CRLF, "$"

;--------------------------------------------------------------------


.code

	extern IEEEtoMSBshort$n:proc, IEEEtoMSBlong$n:proc
	extern MSBtoIEEEshort$n:proc, MSBtoIEEElong$n:proc
	extern hexout$b:proc

.startup

	;-----------------------------------;
	; Print the single precision floats ;
	;-----------------------------------;

	mov 	di, offset shorts	; Point DI to the first short real
	mov 	cx, 5				; Convert 5 short reals
	mov 	si, 4				; Tell prthex we're using short reals

shortLoop:
	PR_STR_HEX newline			; Start new line, print IEEE hex
	call	IEEEtoMSBshort$n	; Convert to MSB format
	jnc 	shortSkip1
	PR_STR	Ishort				; Tell user about conversion error
	jmp 	shortSkip3
shortSkip1:
	PR_STR_HEX cvtsto			; Show user what IEEE # converted to
	call	MSBtoIEEEshort$n	; Convert it back to IEEE
	jnc 	shortSkip2
	PR_STR	Mshort				; Tell user about conversion error
	jmp 	shortSkip3
shortSkip2:
	PR_STR_HEX cvtsto			; Show user it converts back O.K.
shortSkip3:
	add 	di, si				; point to the next float
	loop	shortLoop			; continue until we've done them all

	;------------------------;
	; Convert the long reals ;
	;------------------------;

	; Since array of long reals starts immediately after floats, DI
	; already points to the first long real
	mov 	cx, 5				; Convert 5 long reals
	mov 	si, 8				; Tell prthex we're using long reals

longLoop:
	PR_STR_HEX newline			; Start new line, print IEEE hex
	call	IEEEtoMSBlong$n 	; Convert to MSB format
	jnc 	longSkip1
	PR_STR	Ilong				; Tell user about conversion error
	jmp 	longSkip3
longSkip1:
	PR_STR_HEX cvtsto			; Show user what IEEE # converted to
	call	MSBtoIEEElong$n 	; Convert back to IEEE
	jnc 	longSkip2
	PR_STR	Mlong				; Tell user about conversion error
	jmp 	longSkip3
longSkip2:
	PR_STR_HEX cvtsto			; Show user it converts back O.K.
longSkip3:
	add 	di, si				; Point to the next double
	loop	longLoop			; continue until we've done them all

	.exit	0


;--------------------------------------------------------------------
;	prthex - print a sequence of bytes in hexadecimal, separated by
;	spaces
;
;	INP:	DI - pointer to the bytes to print
;			SI - number of bytes to print
;	USES:	AX, DX
;
prthex:
	@SaveRegs	cx, di

	mov 	cx, si				; # bytes to print in hex
	add 	di, cx				; point to the last byte
errorLoop:
	dec 	di					; Adjust counter
	mov 	al, [di]			; get current byte
	call	hexout$b			; print the byte in hex
	loop	errorLoop			; continue until all bytes printed

	@RestoreRegs
	ret

	end
